//###########################################################################

#include "DSP2833x_Device.h"     // DSP2833x Headerfile Include File
//#include "DSP2833x_Examples.h"   // DSP2833x Examples Include File

//---------------------------------------------------------------------------
// InitI2C: 
//---------------------------------------------------------------------------
// This function initializes the I2C to a known state.
//
//---------------------------------------------------------------------------
// Example: InitI2CGpio: 
//---------------------------------------------------------------------------
// This function initializes GPIO pins to function as I2C pins
//
// Each GPIO pin can be configured as a GPIO pin or up to 3 different
// peripheral functional pins. By default all pins come up as GPIO
// inputs after reset.  
// 
// Caution: 
// Only one GPIO pin should be enabled for SDAA operation.
// Only one GPIO pin shoudl be enabled for SCLA operation. 
// Comment out other unwanted lines.

void InitGpio()
{

   EALLOW;
/* Enable internal pull-up for the selected pins */

	GpioCtrlRegs.GPBPUD.bit.GPIO32 = 0;    // Enable pull-up for GPIO32 (SDAA)
	GpioCtrlRegs.GPBPUD.bit.GPIO33 = 0;	   // Enable pull-up for GPIO33 (SCLA)
	GpioCtrlRegs.GPBPUD.bit.GPIO34 = 0;	   // Enable pull-up for GPIO34 (SQW)
	GpioCtrlRegs.GPAPUD.all = 1;
	GpioCtrlRegs.GPCPUD.all = 1;
	
/* Set qualification for selected pins to asynch only */

	GpioCtrlRegs.GPBQSEL1.bit.GPIO32 = 3;  // Asynch input GPIO32 (SDAA)
    GpioCtrlRegs.GPBQSEL1.bit.GPIO33 = 3;  // Asynch input GPIO33 (SCLA)
//	GpioCtrlRegs.GPCQSEL1.all = 3;  // Asynch input GPIO33 (SCLA)
/* Configure SCI pins using GPIO regs*/

	GpioCtrlRegs.GPBMUX1.bit.GPIO32 = 1;   // Configure GPIO32 for SDAA operation
	GpioCtrlRegs.GPBMUX1.bit.GPIO33 = 1;   // Configure GPIO33 for SCLA operation
	GpioCtrlRegs.GPBMUX1.bit.GPIO34 = 0;   // Configure GPIO34 for SQW O/P
	
/* Configure User I/F pins using GPIO regs */
	GpioCtrlRegs.GPBMUX2.bit.GPIO50 = 0;   // Configure GPIO50 for Red Alarm
	GpioCtrlRegs.GPBMUX2.bit.GPIO54 = 0;   // Configure GPIO54 for Amber Status
	GpioCtrlRegs.GPBMUX2.bit.GPIO51 = 0;   // Configure GPIO51 for Amber Alarm
	GpioCtrlRegs.GPBMUX2.bit.GPIO55 = 0;   // Configure GPIO55 for Green Status
	GpioCtrlRegs.GPBMUX2.bit.GPIO52 = 0;   // Configure GPIO52 for Green Alarm
	//GpioCtrlRegs.GPBMUX2.bit.GPIO49 = 0;   // Configure GPIO21 for B/L Switch  // use this for P_DIODE2 interrupt pin
	GpioCtrlRegs.GPBMUX1.bit.GPIO35 = 0;   // Configure GPIO35 for Buzzer O/P
	GpioCtrlRegs.GPBMUX2.bit.GPIO48 = 0;   // Configure GPIO25 for Alarm Reset
	GpioCtrlRegs.GPBMUX2.bit.GPIO56 = 0;   // Configure GPIO56 for B/L Adjust
	GpioCtrlRegs.GPBMUX2.bit.GPIO53 = 0;   // Configure GPIO53 for Red Status
	
	GpioCtrlRegs.GPCMUX1.bit.GPIO72 = 0;	/* Configure GPIO72 for Reset1 O/P */
	GpioCtrlRegs.GPCMUX1.bit.GPIO73 = 0;	/* Configuer GPIO73 for Reset2 O/P */
	
	GpioCtrlRegs.GPBDIR.bit.GPIO50 = 1;   // Configure GPIO50 for Red Alarm
	GpioCtrlRegs.GPBDIR.bit.GPIO54 = 1;   // Configure GPIO54 for Amber Status
	GpioCtrlRegs.GPBDIR.bit.GPIO51 = 1;   // Configure GPIO51 for Amber Alarm
	GpioCtrlRegs.GPBDIR.bit.GPIO55 = 1;   // Configure GPIO55 for Green Status
	GpioCtrlRegs.GPBDIR.bit.GPIO52 = 1;   // Configure GPIO52 for Green Alarm
	//GpioCtrlRegs.GPBDIR.bit.GPIO49 = 0;   // Configure GPIO21 for B/L Switch  // use this for P_DIODE2 interrupt pin
	GpioCtrlRegs.GPBDIR.bit.GPIO35 = 1;   // Configure GPIO35 for Buzzer O/P
	GpioCtrlRegs.GPBDIR.bit.GPIO48 = 0;   // Configure GPIO25 for Alarm Reset
	GpioCtrlRegs.GPBDIR.bit.GPIO56 = 1;   // Configure GPIO56 for B/L Adjust
	GpioCtrlRegs.GPBDIR.bit.GPIO53 = 1;   // Configure GPIO53 for Red Status	

	GpioCtrlRegs.GPCDIR.bit.GPIO72 = 1; 	/* Configure GPIO72 for O/P */
	GpioCtrlRegs.GPCDIR.bit.GPIO73 = 1;		/* Configure GPIO73 for O/P */

	GpioDataRegs.GPBSET.bit.GPIO50 = 1;   // Configure GPIO50 for Red Alarm
	GpioDataRegs.GPBSET.bit.GPIO54 = 1;   // Configure GPIO54 for Amber Status
	GpioDataRegs.GPBSET.bit.GPIO51 = 1;   // Configure GPIO51 for Amber Alarm
	GpioDataRegs.GPBSET.bit.GPIO55 = 1;   // Configure GPIO55 for Green Status
	GpioDataRegs.GPBSET.bit.GPIO52 = 1;   // Configure GPIO52 for Green Alarm

	GpioDataRegs.GPBSET.bit.GPIO35 = 0;   // Configure GPIO35 for Buzzer O/P
	GpioDataRegs.GPBCLEAR.bit.GPIO35 = 1;   // Configure GPIO35 for Buzzer O/P

	GpioDataRegs.GPBSET.bit.GPIO56 = 1;   // Configure GPIO56 for B/L Adjust
	GpioDataRegs.GPBSET.bit.GPIO53 = 1;   // Configure GPIO53 for Red Status	

	GpioDataRegs.GPCSET.bit.GPIO72 = 1;		/* Configuer GPIO72 for Reset1 */
	GpioDataRegs.GPCSET.bit.GPIO73 = 1;		/* Configure GPIO73 for Reset2 */
/* Configure LCD pins using GPIO regs*/  	
   	
	GpioCtrlRegs.GPBMUX2.bit.GPIO60 = 0;
	GpioCtrlRegs.GPBMUX2.bit.GPIO59 = 0;
	GpioCtrlRegs.GPBMUX2.bit.GPIO58 = 0;

	GpioCtrlRegs.GPAMUX1.bit.GPIO0 = 0;
	GpioCtrlRegs.GPAMUX1.bit.GPIO1 = 0;
	GpioCtrlRegs.GPAMUX1.bit.GPIO2 = 0;
	GpioCtrlRegs.GPAMUX1.bit.GPIO3 = 0;
	GpioCtrlRegs.GPAMUX1.bit.GPIO4 = 0;
	GpioCtrlRegs.GPAMUX1.bit.GPIO5 = 0;
	GpioCtrlRegs.GPAMUX1.bit.GPIO6 = 0;
	GpioCtrlRegs.GPAMUX1.bit.GPIO7 = 0;
	
	/*  ADC */
	GpioCtrlRegs.GPCMUX1.bit.GPIO64 = 0;
	GpioCtrlRegs.GPCMUX1.bit.GPIO65 = 0;
	GpioCtrlRegs.GPCMUX1.bit.GPIO66 = 0;
	GpioCtrlRegs.GPCMUX1.bit.GPIO67 = 0;
	GpioCtrlRegs.GPCMUX1.bit.GPIO68 = 0;
	GpioCtrlRegs.GPCMUX1.bit.GPIO69 = 0;
	GpioCtrlRegs.GPCMUX1.bit.GPIO70 = 0;
	GpioCtrlRegs.GPCMUX1.bit.GPIO71 = 0;


	GpioDataRegs.GPACLEAR.bit.GPIO0 = 1;
	GpioDataRegs.GPACLEAR.bit.GPIO1 = 1;
	GpioDataRegs.GPACLEAR.bit.GPIO2 = 1;	
	GpioDataRegs.GPACLEAR.bit.GPIO3 = 1;
	GpioDataRegs.GPACLEAR.bit.GPIO4 = 1;
	GpioDataRegs.GPACLEAR.bit.GPIO5 = 1;	
	GpioDataRegs.GPACLEAR.bit.GPIO6 = 1;
	GpioDataRegs.GPACLEAR.bit.GPIO7 = 1;
	GpioDataRegs.GPBCLEAR.bit.GPIO60 = 1;
	GpioDataRegs.GPBCLEAR.bit.GPIO59 = 1;
	GpioDataRegs.GPBCLEAR.bit.GPIO58 = 1;	

	
	GpioCtrlRegs.GPBDIR.bit.GPIO60 = 1;			// GPIO60 is an output R/W
	GpioCtrlRegs.GPBDIR.bit.GPIO59 = 1;			// GPIO59 is an output RS
	GpioCtrlRegs.GPBDIR.bit.GPIO58 = 1;			// GPIO58 is an output E
	
	GpioCtrlRegs.GPADIR.bit.GPIO0 = 1;
	GpioCtrlRegs.GPADIR.bit.GPIO1 = 1;
	GpioCtrlRegs.GPADIR.bit.GPIO2 = 1;
	GpioCtrlRegs.GPADIR.bit.GPIO3 = 1;
	GpioCtrlRegs.GPADIR.bit.GPIO4 = 1;
	GpioCtrlRegs.GPADIR.bit.GPIO5 = 1;
	GpioCtrlRegs.GPADIR.bit.GPIO6 = 1;
	GpioCtrlRegs.GPADIR.bit.GPIO7 = 1;
	
	/* ADC */
	GpioCtrlRegs.GPCDIR.bit.GPIO64 = 0;			// GPIO64 is an input
	GpioCtrlRegs.GPCDIR.bit.GPIO65 = 0;
	GpioCtrlRegs.GPCDIR.bit.GPIO66 = 0;
	GpioCtrlRegs.GPCDIR.bit.GPIO67 = 0;
	GpioCtrlRegs.GPCDIR.bit.GPIO68 = 0;
	GpioCtrlRegs.GPCDIR.bit.GPIO69 = 0;
	GpioCtrlRegs.GPCDIR.bit.GPIO70 = 0;
	GpioCtrlRegs.GPCDIR.bit.GPIO71 = 0;

;
/*	GpioDataRegs.GPCCLEAR.bit.GPIO64 = 1;	
	GpioDataRegs.GPCCLEAR.bit.GPIO65 = 1;
	GpioDataRegs.GPCCLEAR.bit.GPIO66 = 1;
	GpioDataRegs.GPCCLEAR.bit.GPIO67 = 1;
	GpioDataRegs.GPCCLEAR.bit.GPIO68 = 1;
	GpioDataRegs.GPCCLEAR.bit.GPIO69 = 1;
	GpioDataRegs.GPCCLEAR.bit.GPIO70 = 1;
	GpioDataRegs.GPCCLEAR.bit.GPIO71 = 1;  */
	
/* Configure CMOS Array pins using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX1.bit.GPIO43 = 0;	// Configure GPIO43
	GpioCtrlRegs.GPBDIR.bit.GPIO43 = 1;		// GPIO43 is ST
	GpioCtrlRegs.GPBMUX1.bit.GPIO42 = 0;	// Configure GPIO42
	GpioCtrlRegs.GPBDIR.bit.GPIO42 = 1;		// GPIO42 is an output CAPT_CLK that clocks the Linear Arrays at upto 5MHz (currently 625KHz)
//	GpioCtrlRegs.GPAMUX2.bit.GPIO30 = 0;	// Configure GPIO30  ************  not used now  *********
//	GpioCtrlRegs.GPADIR.bit.GPIO30 = 1;		// GPIO30 is an output CLK_Array
	GpioDataRegs.GPBSET.bit.GPIO43 = 1;
	GpioDataRegs.GPBCLEAR.bit.GPIO42 = 1;
//	GpioDataRegs.GPASET.bit.GPIO30 = 1;	

/* Configure Photodiode1 pin using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX2.bit.GPIO61 = 0;	// Configure GPIO61 P_DIODE1 interrupt 
	GpioCtrlRegs.GPBDIR.bit.GPIO61 = 0;		// GPIO61 is an input	
	
/* Configure Photodiode2 pin using GPIO regs 	G.Lawson added 10/02/12*/
	GpioCtrlRegs.GPBMUX2.bit.GPIO49 = 0;	// Configure GPIO49 as P_DIODE2 interrupt P400/5	
	GpioCtrlRegs.GPBDIR.bit.GPIO49 = 0;		// GPIO49 is an input	
	
/* Configure ADC Power Down Pins using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX2.bit.GPIO62 = 0;	// Configure GPIO62  ADC_PD1 enables
	GpioCtrlRegs.GPBDIR.bit.GPIO62 = 1;		// GPIO62 is an output	
	GpioDataRegs.GPBSET.bit.GPIO62 = 1;		// Initialise High
	GpioCtrlRegs.GPBMUX2.bit.GPIO63 = 0;	// Configure GPIO63  ADC_PD1
	GpioCtrlRegs.GPBDIR.bit.GPIO63 = 1;		// GPIO63 is an output	
	GpioDataRegs.GPBSET.bit.GPIO63 = 1;		// Initialise High

/* Configure Speaker Control Pins using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX1.bit.GPIO36 = 0;	// Configure GPIO36  BSEL0
	GpioCtrlRegs.GPBDIR.bit.GPIO36 = 1;		// GPIO36 is an output	

	GpioCtrlRegs.GPBMUX1.bit.GPIO37 = 0;	// Configure GPIO37  BSEL1
	GpioCtrlRegs.GPBDIR.bit.GPIO37 = 1;		// GPIO37 is an output	
	
	GpioCtrlRegs.GPBMUX1.bit.GPIO38 = 0;	// Configure GPIO38  BSEL2
	GpioCtrlRegs.GPBDIR.bit.GPIO38 = 1;		// GPIO38 is an output	

	GpioCtrlRegs.GPBMUX1.bit.GPIO41 = 0;	// Configure GPIO42  SPKR_EN
	GpioCtrlRegs.GPBDIR.bit.GPIO41 = 1;		// GPIO42 is an output	
	GpioDataRegs.GPBCLEAR.bit.GPIO41 = 1;	// Initialise Low	
	
//	GpioDataRegs.GPBCLEAR.bit.GPIO36 = 1;	// Initialise Low
	GpioDataRegs.GPBSET.bit.GPIO36 = 1;		// Initialise High	
	GpioDataRegs.GPBCLEAR.bit.GPIO37 = 1;	// Initialise Low
//	GpioDataRegs.GPBSET.bit.GPIO37 = 1;		// Initialise High
	GpioDataRegs.GPBCLEAR.bit.GPIO38 = 1;	// Initialise Low
//	GpioDataRegs.GPBSET.bit.GPIO38 = 1;		// Initialise High


/* Configure Linear Array PSU using GPIO regs*/  	
	GpioCtrlRegs.GPCMUX2.bit.GPIO81 = 0;	// Configure GPIO81  LA_SHDN
	GpioCtrlRegs.GPCDIR.bit.GPIO81 = 1;		// GPIO81 is an output	
	GpioDataRegs.GPCSET.bit.GPIO81 = 1;		// Initialise High

/* Configure Pump PSU using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX1.bit.GPIO39 = 0;	// Configure GPIO39  PUMP_SHDN
	GpioCtrlRegs.GPBDIR.bit.GPIO39 = 1;		// GPIO39 is an output	
	GpioDataRegs.GPBSET.bit.GPIO39 = 0;		// Initialise Low

/* Configure Laser PSU using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX1.bit.GPIO40 = 0;	// Configure GPIO40  LD_SHDN
	GpioCtrlRegs.GPBDIR.bit.GPIO40 = 1;		// GPIO40 is an output	
	GpioDataRegs.GPBSET.bit.GPIO40 = 1;		// Initialise High

/* Configure VIB PSU using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX2.bit.GPIO57 = 0;	// Configure GPIO57  VIB_EN
	GpioCtrlRegs.GPBDIR.bit.GPIO57 = 1;		// GPIO57 is an output	
	GpioDataRegs.GPBCLEAR.bit.GPIO57 = 1;	// Initialise Low

/* Configure EEPROM WP using GPIO regs*/  	
	GpioCtrlRegs.GPCMUX2.bit.GPIO80 = 0;	// Configure GPIO80  I2C_WP
	GpioCtrlRegs.GPCDIR.bit.GPIO80 = 1;		// GPIO80 is an output	
	GpioDataRegs.GPCSET.bit.GPIO80 = 1;		// Initialise High

/* Configure Alarm Inputs using GPIO regs*/  	
	GpioCtrlRegs.GPBMUX1.bit.GPIO44 = 0;	// Configure GPIO44  Vmon_ALM
	GpioCtrlRegs.GPBDIR.bit.GPIO44 = 0;		// GPIO44 is an input
	GpioCtrlRegs.GPBMUX1.bit.GPIO45 = 0;	// Configure GPIO45  BATT_LOW
	GpioCtrlRegs.GPBDIR.bit.GPIO45 = 0;		// GPIO45 is an input
	GpioCtrlRegs.GPBMUX1.bit.GPIO46 = 0;	// Configure GPIO46  Pump_Low
	GpioCtrlRegs.GPBDIR.bit.GPIO46 = 0;		// GPIO46 is an input
	GpioCtrlRegs.GPBMUX1.bit.GPIO47 = 0;	// Configure GPIO47  Pump_High
	GpioCtrlRegs.GPBDIR.bit.GPIO47 = 0;		// GPIO47 is an input
	GpioCtrlRegs.GPBPUD.bit.GPIO44 = 0;	    // Enable pull-up for Vmon_ALM
	GpioCtrlRegs.GPBPUD.bit.GPIO45 = 0;	    // Enable pull-up for BATT_LOW


    EDIS;    
    
}


	
//===========================================================================
// End of file.
//===========================================================================
