/*-----------------------------------------------------------------------*/
/* MMC/SDC (in SPI mode) control module  (C)ChaN, 2007                   */
/*-----------------------------------------------------------------------*/
/* Only rcvr_spi(), xmit_spi(), disk_timerproc() and some macros         */
/* are platform dependent.                                               */
/*-----------------------------------------------------------------------*/

/*
 * This file was modified from a sample available from the FatFs
 * web site. It was modified to work with a Stellaris DK-LM3S9B96 board.
 */

#include "DSP28x_Project.h"
#include "diskio_ff9a.h"
#include "SD.h"
#include "stdbool.h"
#include "Alert_Globals.h"
/*
 * The following header defines the hardware connections used to connect
 * the SDCard.  This can be found under the relevant board directory.
 */

/* Definitions for MMC/SDC command */
#define CMD0    (0x40+0)    	/* GO_IDLE_STATE */
#define CMD1    (0x40+1)    	/* SEND_OP_COND */
#define CMD8    (0x40+8)    	/* SEND_IF_COND */
#define CMD9    (0x40+9)    	/* SEND_CSD */
#define CMD10    (0x40+10)    	/* SEND_CID */
#define CMD12    (0x40+12)    	/* STOP_TRANSMISSION */
#define CMD16    (0x40+16)    	/* SET_BLOCKLEN */
#define CMD17    (0x40+17)   	/* READ_SINGLE_BLOCK */
#define CMD18    (0x40+18)    	/* READ_MULTIPLE_BLOCK */
#define CMD23    (0x40+23)    	/* SET_BLOCK_COUNT */
#define CMD24    (0x40+24)    	/* WRITE_BLOCK */
#define CMD25    (0x40+25)    	/* WRITE_MULTIPLE_BLOCK */
#define CMD41    (0x40+41)    	/* SEND_OP_COND (ACMD) */
#define CMD55    (0x40+55)    	/* APP_CMD */
#define CMD58    (0x40+58)    	/* READ_OCR */

// asserts the CS pin to the card
static
void SELECT (void)
{
	GpioDataRegs.GPACLEAR.bit.GPIO19 = 1;	/* CS_LOW */
}

// de-asserts the CS pin to the card.
static
void DESELECT (void)
{
	GpioDataRegs.GPASET.bit.GPIO19 = 1;		/* CS_HIGH */
}

/*--------------------------------------------------------------------------

   Module Private Functions

---------------------------------------------------------------------------*/

static volatile
DSTATUS Stat = STA_NOINIT;    			/* Disk status */

static volatile
BYTE Timer1, Timer2;    				/* 100Hz decrement timer */

static
BYTE CardType;            				/* b0:MMC, b1:SDC, b2:Block addressing */

static
BYTE PowerFlag = 0;     				/* indicates if "power" is on */

/*-----------------------------------------------------------------------*/
/* Transmit a byte to MMC via SPI  (Platform dependent)                  */
/*-----------------------------------------------------------------------*/

static 
void xmit_spi (BYTE dat) 
{ 
//    DWORD rcvdat;
//
//    SPIDataPut(&SpiaRegs, dat); 						/* Write the data to the tx fifo */
//
//    SPIDataGet(&SpiaRegs, (unsigned int *) &rcvdat); 	/* flush data read during the write */

	
	SpiaRegs.SPITXBUF = dat << 8;         				/* Transmit Byte */
	/* while(SpiaRegs.SPISTS.bit.INT_FLAG != 1); */ 	/* Wait until the RXBUF has received last bit */
  	asm(" nop");
	asm(" nop");
  	while(SpiaRegs.SPISTS.bit.INT_FLAG != 1);
  	asm(" nop");
	asm(" nop");
  														/* Read Byte from RXBUF and return */
} 

/*-----------------------------------------------------------------------*/
/* Receive a byte from MMC via SPI  (Platform dependent)                 */
/*-----------------------------------------------------------------------*/
static 
 BYTE rcvr_spi (void) 
{ 

	BYTE ret; 
  	SpiaRegs.SPITXBUF = 0xFF00;        				/* Transmit DUMMY Byte */
	while(SpiaRegs.SPISTS.bit.INT_FLAG == 0){}; 	/* Wait until the RXBUF has received last bit */
	ret = (SpiaRegs.SPIRXBUF);						/*Read Byte from RXBUF and return */
	return ret; 									

}

static
void rcvr_spi_m (BYTE *dst)
{
    *dst = rcvr_spi();
}

/*-----------------------------------------------------------------------*/
/* Wait for card ready                                                   */
/*-----------------------------------------------------------------------*/

static
BYTE wait_ready (void)
{
    BYTE res;
    Timer2 = 50;    							/* Wait for ready in timeout of 500ms */
    rcvr_spi();
    do 
        res = rcvr_spi();
    while ((res != 0xFF) && Timer2); 
    return res;
}
/*-----------------------------------------------------------------------*/
/* Send 80 or so clock transitions with CS and DI held high. This is     */
/* required after card power up to get it into SPI mode                  */
/*-----------------------------------------------------------------------*/
static
void send_initial_clock_train (void)
{
    unsigned int i;
   
    /* DESELECT(); */
	CS_HIGH;								/* Ensure CS is held high. */
   
    /* Send 10 bytes over the SSI. This causes the clock to wiggle the */
    /* required number of times. */
    for(i = 0 ; i < 10 ; i++)
    {
        /* Write DUMMY data. SSIDataPut() waits until there is room in the */
        /* FIFO. */
		rcvr_spi();
    }
   
}

/*-----------------------------------------------------------------------*/
/* Power Control  (Platform dependent)                                   */
/*-----------------------------------------------------------------------*/
/* When the target system does not support socket power control, there   */
/* is nothing to do in these functions and chk_power always returns 1.   */

static
void power_on (void)
{
	
	
    /*
     * This doesn't really turn the power on, but initializes the
     * SSI port and pins needed to talk to the card.
     */
     
     /*EALLOW; */

    /* Enable the peripherals used to drive the SDC on SPI */

	/*SysCtrlRegs.PCLKCR0.bit.SPIAENCLK = 1; */


    /*
     * Configure the appropriate pins to be SSI instead of GPIO. The CS
     * signal is directly driven to ensure that we can hold it low through a
     * complete transaction with the SD card.
     */
     //TODO: Board Specific Pin Configuration
     //GPIO54 - MOSI - PER1
     //GPIO55 - MISO - PER1
     //GPIO56 - CLK - PER1
     //GPIO57 - CS - GPIO
     /*GpioCtrlRegs.GPBMUX2.bit.GPIO54 = 1; */
     /*GpioCtrlRegs.GPBDIR.bit.GPIO54 = 1; */
     //GpioCtrlRegs.GPBQSEL2.bit.GPIO54 = 3;
     
    /* GpioCtrlRegs.GPBMUX2.bit.GPIO55 = 1; */
    /* GpioCtrlRegs.GPBDIR.bit.GPIO55 = 0; */
    /* GpioCtrlRegs.GPBQSEL2.bit.GPIO55 = 3; */    
     
    /* GpioCtrlRegs.GPBMUX2.bit.GPIO56 = 1; */
    /* GpioCtrlRegs.GPBDIR.bit.GPIO56 = 1; */
     //GpioCtrlRegs.GPBQSEL2.bit.GPIO56 = 3;     
     
    /* GpioCtrlRegs.GPBMUX2.bit.GPIO57 = 0; */
    /* GpioCtrlRegs.GPBDIR.bit.GPIO57 = 1; */
     //GpioCtrlRegs.GPBQSEL2.bit.GPIO57 = 3;    
     //CS High
    /* GpioDataRegs.GPBDAT.bit.GPIO57 = 1; */
     
    
//     GpioDataRegs.GPBDAT.bit.GPIO54 = 1;
//     GpioDataRegs.GPBDAT.bit.GPIO55 = 1;
//     GpioDataRegs.GPBDAT.bit.GPIO56 = 1;
//     GpioDataRegs.GPBDAT.bit.GPIO57 = 1;
//     GpioDataRegs.GPBDAT.bit.GPIO54 = 0;
//     GpioDataRegs.GPBDAT.bit.GPIO55 = 0;
//     GpioDataRegs.GPBDAT.bit.GPIO56 = 0;
//     GpioDataRegs.GPBDAT.bit.GPIO57 = 0;
       
    /* Deassert the SSI0 chip selects for both the SD card and serial flash */
//    GPIOPinWrite(SDCARD_CS_BASE, SDCARD_CS_PIN, SDCARD_CS_PIN);
//    GPIOPinWrite(SFLASH_CS_BASE, SFLASH_CS_PIN, SFLASH_CS_PIN);

    /* Configure the SSI0 port */
    /* SysCtrlRegs.LOSPCP.all = 0; */
    /* SpiaRegs.SPIBRR = 0 ; */
    
    //InitSpi(&SpiaRegs, SPI_CR_SW_RESET | SPI_CR_CHAR_8 | SPI_CR_CLKPOL_RIS, SPI_CTL_MASTER | SPI_CTL_TALK, 0);
    //InitSpi(&SpiaRegs, SPI_CR_SW_RESET | SPI_CR_CHAR_8 | SPI_CR_CLKPOL_FAL, SPI_CTL_MASTER | SPI_CTL_TALK, 0);
   /* InitSpi(&SpiaRegs,  SPI_CR_SW_RESET | SPI_CR_CHAR_8 | SPI_CR_CLKPOL_RIS, SPI_CTL_MASTER | SPI_CTL_TALK | SPI_CTL_CLKPHS_90, 0x7F); */
    //InitSpi(&SpiaRegs, SPI_CR_SW_RESET | SPI_CR_CHAR_8 | SPI_CR_CLKPOL_FAL, SPI_CTL_MASTER | SPI_CTL_TALK | SPI_CTL_CLKPHS_90, 0);
    //TODO: Set bit rate 
    //No divide on low speed clock


    
//    SSIConfigSetExpClk(SDC_SSI_BASE, SysCtlClockGet(), SSI_FRF_MOTO_MODE_0,
//                       SSI_MODE_MASTER, 400000, 8);
//    SSIEnable(SDC_SSI_BASE);

    /* Set DI and CS high and apply more than 74 pulses to SCLK for the card */
    /* to be able to accept a native command. */
   send_initial_clock_train();

   PowerFlag = 1;
}

// set the SSI speed to the max setting
static
void set_max_speed(void)
{
   
	SpiaRegs.SPIBRR = 0x001F;		/* set for 1.2MHz */
}

static
void power_off (void)
{
   /* PowerFlag = 0; */
}

static
int chk_power(void)        /* Socket power state: 0=off, 1=on */
{
   return PowerFlag;
}

/*-----------------------------------------------------------------------*/
/* Receive a data packet from MMC                                        */
/*-----------------------------------------------------------------------*/
static
BOOL rcvr_datablock (					/* was BOOL instead of int */
    BYTE *buff,            				/* Data buffer to store received data */
    UINT btr            				/* Byte count (must be even number) */
)
{
    BYTE token;
    Timer1 = 10;
   do {                            		/* Wait for data packet in timeout of 100ms */
        token = rcvr_spi();
   } while ((token == 0xFF) && Timer1);
    if(token != 0xFE) return FALSE;    	/* If not valid data token, return with error */
    do {                            	/* Receive the data block into buffer */
        	rcvr_spi_m(buff++);
        	rcvr_spi_m(buff++);
    	} while (btr -= 2);
    rcvr_spi();                        	/* Discard CRC */
    rcvr_spi();
    return TRUE;                    	/* Return with success */
}
/*-----------------------------------------------------------------------*/
/* Send a data packet to MMC                                             */
/*-----------------------------------------------------------------------*/

#if _READONLY == 0
static
BOOL xmit_datablock (						/* was BOOL instead of int GL*/
    const BYTE *buff,    					/* 512 byte data block to be transmitted */
    BYTE token            					/* Data/Stop token */
)
{
    Uint16 j;
    
    BYTE resp = 0;
    if (wait_ready() != 0xFF) return FALSE;
    
    if (token != 0xFD)
    {
    	spi_xmit_byte_FATS(token); 					/* Xmit data token */
   		for(j=0;j<512;j++)
		spi_xmit_byte_FATS(*buff++);
		
   		spi_xmit_byte_FATS(0xFF);					/* transmit 0xFF twice more to send CRC */		
		spi_xmit_byte_FATS(0xFF);
		while (resp != 0xE500)					/* data accepted ok */
		{
			resp = spi_xmit_byte_FATS(0xFF);	
			/*resp &= 0x0F;*/						/* mask response to test for errors */
			if (resp != 0xE500)
			return FALSE;
			/*resp = rcvr_spi();*/
   		/*	if ((resp & 0x0F) != 0x05)	*/			/* was 0x1F GL */
            	/*return FALSE;*/
		}
    }
   return TRUE;
}
#endif
  
/*-----------------------------------------------------------------------*/
/* Send a command packet to MMC                                          */
/*-----------------------------------------------------------------------*/

static
BYTE send_cmd (
    BYTE cmd,        /* Command byte */
    DWORD arg        /* Argument */
)
{
    BYTE n, res;  
	 if (wait_ready() != 0xFF) return 0xFF;	
                     							/* Command */   
    spi_xmit_byte_FATS(cmd);					/* Argument[31..24] */       
    spi_xmit_byte_FATS((BYTE)(arg >> 24));		/* Argument[23..16] */
    spi_xmit_byte_FATS((BYTE)(arg >> 16));		/* Argument[15..8] */   
    spi_xmit_byte_FATS((BYTE)(arg >> 8)); 		/* Argument[7..0] */
    spi_xmit_byte_FATS((BYTE)(arg));
    n = 0;								
    if (cmd == CMD0) n = 0x95;            		/* CRC for CMD0(0)  GO_IDLE_STATE */
    if (cmd == CMD8) n = 0x87;            		/* CRC for CMD8(0x1AA) SEND_IF_COMMAND */
    spi_xmit_byte_FATS(n);			
    /* Receive command response */
    if (cmd == CMD12) rcvr_spi();        		/* Skip a stuff byte when stop reading */
    	n = 10;                                		/* Wait for a valid response in timeout of 10 attempts */
    do
        res = rcvr_spi();
   while ((res & 0x80) && --n);  				/* bit wise AND */						
    return res;            						/* Return with the response value */
}

/*--------------------------------------------------------------------------

   Public Functions

---------------------------------------------------------------------------*/


/*-----------------------------------------------------------------------*/
/* Initialize Disk Drive                                                 */
/*-----------------------------------------------------------------------*/

DSTATUS disk_initialize 
(
    BYTE drv        						/* Physical drive nmuber (0) */
)
{
    BYTE n, received_result,ty, ocr[4];
	BYTE spi_reply,a_ocr,b_ocr,c_ocr,d_ocr;

    if (drv) return STA_NOINIT;            				/* Supports only single drive */
    if (Stat & STA_NODISK) return Stat;    				/* No card in the socket */

   power_on();                             				/* Force socket power on this function alreday calls */ 												/* send_initial_clock_train */
   send_initial_clock_train(); 		 					/* send 80 clock pulses with CS_HIGH */
    CS_LOW;												/* CS = L  enable SD card */
   ty = 0;
   
   spi_reply = (send_cmd(CMD0, 0));						/* test response for 0x01 */
   if (spi_reply == 1) 									/* send CMD0 to reset media (CMD0 =0x4000 GO_IDLE_STATE) */
   {            										/* the response should be 0x01 if successful else it failed */
   														/* Enter Idle state */
        Timer1 = 100;                        			/* Initialization timeout of 1000 msec */
   														/* CMD8 = 0x4800 SEND_IF_COND */
        if (send_cmd(CMD8, 0x1AA) == 1) 				/* SDC Ver2+ */	
    	{    											
           ocr[0] = rcvr_spi();
           a_ocr = ocr[0];
           ocr[1] = rcvr_spi();
           b_ocr = ocr[1];
           ocr[2] = rcvr_spi();
           c_ocr = ocr[2];
           ocr[3] = rcvr_spi();
           d_ocr = ocr[3];
            if (ocr[2] == 0x01 && ocr[3] == 0xAA) 		/* The card can work at vdd range of 2.7-3.6V */
    		{    										
                do{
  			 											/* CMD55 = 0x7700 */
                    if (send_cmd(CMD55, 0) <= 1 && send_cmd(CMD41, 1UL << 30) == 0)    break;    /* ACMD41 with HCS bit */
                } while (Timer1); 
                if (Timer1 && send_cmd(CMD58, 0) == 0) 
    			{    												/* Check CCS bit */
                    for (n = 0; n < 4; n++) ocr[n] = rcvr_spi();
                    ty = (ocr[0] & 0x40) ? 6 : 2;
                }
            }
        } else 
        {                            											/* SDC Ver1 or MMC */
            ty = (send_cmd(CMD55, 0) <= 1 && send_cmd(CMD41, 0) <= 1) ? 2 : 1;  /* SDC : MMC */
            do {
                if (ty == 2) 
                {
                    if (send_cmd(CMD55, 0) <= 1 && send_cmd(CMD41, 0) == 0) break;    /* ACMD41 */
                } 
                else 
                {
                    if (send_cmd(CMD1, 0) == 0) break;                                /* CMD1 */
                }
            } while (Timer1);
            if (!Timer1 || send_cmd(CMD16, 512) != 0)    			/* Select R/W block length */
                ty = 0;
        }
    }
    CardType = ty; 
   
   /* DESELECT(); */           								/* CS = H */
   CS_HIGH;
    received_result = rcvr_spi();           				/* Idle (Release DO) */
    												
    if (ty) 
    {            											/* Initialization succeded */
        Stat &= ~STA_NOINIT;        						/* Clear STA_NOINIT */
        set_max_speed();  
    } 
    else 
    {            											/* Initialization failed */
        power_off();
    }

    return Stat;
}

/*-----------------------------------------------------------------------*/
/* Get Disk Status                                                       */
/*-----------------------------------------------------------------------*/

DSTATUS disk_status (
    BYTE drv        /* Physical drive nmuber (0) */
)
{
    if (drv) return STA_NOINIT;        /* Supports only single drive */
    return Stat;
}

/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */
/*-----------------------------------------------------------------------*/

DRESULT disk_read (
    BYTE drv,            					/* Physical drive nmuber (0) */
    BYTE *buff,            					/* Pointer to the data buffer to store read data */
    DWORD sector,        					/* Start sector number (LBA) */
    BYTE count            					/* Sector count (1..255) */
)
{
    if (drv || !count) return RES_PARERR;
    if (Stat & STA_NOINIT) return RES_NOTRDY;

    if (!(CardType & 4)) sector *= 512;    	/* Convert to byte address if needed */

   /* SELECT(); */           				/* CS = L */
    CS_LOW;
    if (count == 1) 						/* Single block read */
    {    						
        if ((send_cmd(CMD17, sector) == 0)  /* READ_SINGLE_BLOCK */
            && rcvr_datablock(buff, 512))   
            count = 0; 
    }
    else 
    {                							/* Multiple block read */
        if (send_cmd(CMD18, sector) == 0)  		/* READ_MULTIPLE_BLOCK */
        {    
            do {
                if (!rcvr_datablock(buff, 512)) break;
                buff += 512;
            } while (--count);
            send_cmd(CMD12, 0);                	/* STOP_TRANSMISSION */
        }
    }

   /* DESELECT(); */           /* CS = H */
   CS_HIGH;
    rcvr_spi();            						/* Idle (Release DO) */

    return count ? RES_ERROR : RES_OK;			/* RES_OK = 0, RES_ERROR = 1 */
}

/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */
/*-----------------------------------------------------------------------*/

#if _READONLY == 0
DRESULT disk_write (
    BYTE drv,            /* Physical drive nmuber (0) */
    const BYTE *buff,    /* Pointer to the data to be written */
    DWORD sector,        /* Start sector number (LBA) */
    BYTE count           /* Sector count (1..255) */
)
{
    if (drv || !count) return RES_PARERR;
    if (Stat & STA_NOINIT) return RES_NOTRDY;
    if (Stat & STA_PROTECT) return RES_WRPRT;

    if (!(CardType & 4)) sector *= 512;    /* Convert to byte address if needed */

    SELECT();            /* CS = L */

    if (count == 1) {    /* Single block write */
        if ((send_cmd(CMD24, sector) == 0)    /* WRITE_BLOCK */
            && xmit_datablock(buff, 0xFE))		/* send START BLOCK token byte */
            count = 0;
    }
    else {                /* Multiple block write */
        if (CardType & 2) {
            send_cmd(CMD55, 0); send_cmd(CMD23, count);    /* ACMD23 */
        }
        if (send_cmd(CMD25, sector) == 0) {    /* WRITE_MULTIPLE_BLOCK */
            do {
                if (!xmit_datablock(buff, 0xFC)) break;
                buff += 512;
            } while (--count);
            if (!xmit_datablock(0, 0xFD))    /* STOP_TRAN token */
                count = 1;
        }
    }

    DESELECT();            /* CS = H */
    rcvr_spi();            /* Idle (Release DO) */

    return count ? RES_ERROR : RES_OK;
}
#endif /* _READONLY */

/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */
/*-----------------------------------------------------------------------*/

DRESULT disk_ioctl (
    BYTE drv,        /* Physical drive nmuber (0) */
    BYTE ctrl,        /* Control code */
    void *buff        /* Buffer to send/receive control data */
)
{
    DRESULT res;
    BYTE n, csd[16], *ptr = buff;
    WORD csize;

    if (drv) return RES_PARERR;
    	res = RES_ERROR;
     {
        if (Stat & STA_NOINIT) return RES_NOTRDY;
        SELECT();        /* CS = L */
        switch (ctrl) {
        case GET_SECTOR_COUNT :    /* Get number of sectors on the disk (DWORD) */
            if ((send_cmd(CMD9, 0) == 0) && rcvr_datablock(csd, 16)) {
                if ((csd[0] >> 6) == 1) {    /* SDC ver 2.00 */
                    csize = csd[9] + ((WORD)csd[8] << 8) + 1;
                    *(DWORD*)buff = (DWORD)csize << 10;
                } else {                    /* MMC or SDC ver 1.XX */
                    n = (csd[5] & 15) + ((csd[10] & 128) >> 7) + ((csd[9] & 3) << 1) + 2;
                    csize = (csd[8] >> 6) + ((WORD)csd[7] << 2) + ((WORD)(csd[6] & 3) << 10) + 1;
                    *(DWORD*)buff = (DWORD)csize << (n - 9);
                }
                res = RES_OK;
            }
            break;

        case GET_SECTOR_SIZE :    /* Get sectors on the disk (WORD) */
            *(WORD*)buff = 512;
            res = RES_OK;
            break;

        case CTRL_SYNC :    /* Make sure that data has been written */
            if (wait_ready() == 0xFF)
                res = RES_OK;
            break;
		case MMC_GET_CSD:	/* Receive CSD as a data block (16 bytes) */
        	if (send_cmd(CMD9,0) == 0
        		&& rcvr_datablock(ptr,16))
        		res = RES_OK;
        	break;
        case MMC_GET_CID:	/* Receive CID as a data block (16 bytes) */
        	if (send_cmd(CMD10,0) == 0
        		&& rcvr_datablock(ptr,16))
        		res = RES_OK;
        		break;
        case MMC_GET_OCR:	/* Receive OCR as an R3 response (4 bytes) */
        	if (send_cmd(CMD58,0) == 0)
        	{
        		for (n = 0; n < 4;n++)
        			*ptr++ = rcvr_spi();
        		res = RES_OK;
        	}

        default:
            res = RES_PARERR;
        }
        DESELECT();            /* CS = H */
        rcvr_spi();            /* Idle (Release DO) */
    }

    return res;
}

/*-----------------------------------------------------------------------*/
/* Device Timer Interrupt Procedure  (Platform dependent)                */
/*-----------------------------------------------------------------------*/
/* This function must be called in period of 10ms                        */

/*void disk_timerproc (void)
{
//    BYTE n, s;
    BYTE n;


    n = Timer1;                        
    if (n) Timer1 = --n;
    n = Timer2;
    if (n) Timer2 = --n;

}*/
void disk_timerproc (void)
{
    static BYTE pv;
    BYTE n;


    n = Timer1;                        /* 100Hz decrement timer */
    if (n) Timer1 = --n;
    n = Timer2;
    if (n) Timer2 = --n;
 
    /* Code from mmc-c28x.c stops here */
  
   /* n = pv; */
   /* pv = SOCKPORT & (SOCKWP | SOCKINS); */   /* Sample socket switch */

   /* if (n == pv) {  */                  	/* Have contacts stabled? */
   /*     s = Stat; */

   /*     if (pv & SOCKWP) */           	/* WP is H (write protected) */
   /*         s |= STA_PROTECT; */
   /*     else    */                    	/* WP is L (write enabled) */
   /*         s &= ~STA_PROTECT; */			/* this is the critical line that modifies Stat so that the card is not detected as write protected */

   /*     if (pv & SOCKINS)  */          	/* INS = H (Socket empty) */
   /*         s |= (STA_NODISK | STA_NOINIT); */
   /*     else      */                  	/* INS = L (Card inserted) */
   /*         s &= ~STA_NODISK; */

   /*     Stat = s; */
   /* } */
}

/*---------------------------------------------------------*/
/* User Provided Timer Function for FatFs module           */
/*---------------------------------------------------------*/
/* This is a real time clock service to be called from     */
/* FatFs module. Any valid time must be returned even if   */
/* the system does not support a real time clock.          */
//Return Value
 
// Current time is returned packed into a DWORD value. 
// The bit field is as follows:
// bit31:25 Year from 1980 (0..127) 
// bit24:21 Month (1..12) 
// bit20:16 Day in month(1..31) 
// bit15:11 Hour (0..23) 
// bit10:5 Minute (0..59) 
// bit4:0 Second / 2 (0..29)
DWORD get_fattime (void)
{
	DWORD FAT_time; 
	unsigned long FAT_tenths;
	unsigned long FAT_year;
    unsigned long FAT_month;
    unsigned long FAT_day;				/* read but not used */
    unsigned long FAT_date;
    unsigned long FAT_hours;
    unsigned long FAT_minutes;
    unsigned long FAT_seconds;
    unsigned long FAT_tenths_ul;
	unsigned long FAT_year_ul;
    unsigned long FAT_month_ul;
    unsigned long FAT_date_ul;
    unsigned long FAT_hours_ul;
    unsigned long FAT_minutes_ul;
    unsigned long FAT_seconds_ul;
	//***** M41T81 requires a double slave address, so send first with register address *****//  
   			I2caRegs.I2CCNT 	= 1;					//just write the start register address
			I2caRegs.I2CDXR		= 0x00;					//which is 00h
	 		I2caRegs.I2CMDR.all = 0x6620;				//command byte (Master Tx)
			while(I2caRegs.I2CSTR.bit.ARDY == 0);	
//*****	Read the time and date registers *****//		
			I2caRegs.I2CCNT		= 0x08;					// read 8 bytes (time and date bytes)
			I2caRegs.I2CMDR.all = 0x6c20;				// command byte (Master Rx)

			while(I2caRegs.I2CFFRX.bit.RXFFINT == 0);	// wait for Rx FIFO interrupt
			DelayUs(100);
			I2caRegs.I2CFFRX.bit.RXFFINTCLR = 1;		// clear inetrrupt
			DelayUs(100);	
			FAT_tenths_ul = I2caRegs.I2CDRR;			// read tenths
			FAT_seconds = I2caRegs.I2CDRR;				// read seconds	
			FAT_seconds_ul = FAT_BCD2DEC(FAT_seconds);
			FAT_minutes = I2caRegs.I2CDRR;				// read minutes
			FAT_minutes_ul = FAT_BCD2DEC(FAT_minutes);
			FAT_hours = I2caRegs.I2CDRR;				// read hours
			FAT_hours_ul = FAT_BCD2DEC(FAT_hours);
			FAT_day = I2caRegs.I2CDRR;					// read days
			FAT_date = I2caRegs.I2CDRR;					// read date
			FAT_date_ul = FAT_BCD2DEC(FAT_date);
			FAT_month = I2caRegs.I2CDRR;				// read months
			FAT_month_ul = FAT_BCD2DEC(FAT_month);
			FAT_year = I2caRegs.I2CDRR;					// read years
			FAT_year_ul = FAT_BCD2DEC(FAT_year);

			
			FAT_time = (((FAT_year_ul + 2000)-1980) << 25)    	/* Year = 2012 */
            | (FAT_month_ul << 21)            					/* Month = October */
            | (FAT_date_ul << 16)            					/* Day = 11 */
            | (FAT_hours_ul << 11)            					/* Hour = 11 */
            | (FAT_minutes_ul << 5)            					/* Min = 0 */
            | (FAT_seconds_ul >> 1)                				/* Sec = 0 */
            ;
	
    return  FAT_time;
    
    
}
